#import "UIViewController+Swizzling.h"
#import <objc/runtime.h>

/*
 * The only way found to recognize if view controller hides statusbar was to swizzle.
 * Replace this code if another method is found.
 */
@implementation UIViewController (Swizzling)

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wundeclared-selector"

+ (void)load
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        Class class = [self class];

        SEL defaultSelector = @selector(_preferredStatusBarVisibility);
        SEL swizzledSelector = @selector(swizzled_preferredStatusBarVisibility);

        Method defaultMethod = class_getInstanceMethod(class, defaultSelector);
        Method swizzledMethod = class_getInstanceMethod(class, swizzledSelector);

        BOOL doMethodExists = !class_addMethod(class, defaultSelector, method_getImplementation(swizzledMethod), method_getTypeEncoding(swizzledMethod));

        if (doMethodExists) {
            method_exchangeImplementations(defaultMethod, swizzledMethod);
        }
        else {
            class_replaceMethod(class, swizzledSelector, method_getImplementation(defaultMethod), method_getTypeEncoding(defaultMethod));
        }
    });
}

const int VISIBILITY_HIDDEN = 1;

/*
 * Swizzle to detect if new view controller changes the status bar visiblity.
 */
- (int)swizzled_preferredStatusBarVisibility
{
    int result = [self swizzled_preferredStatusBarVisibility];

    // Delay just enough for UIApplication.shared.statusBarFrame to be updated.
    dispatch_async(dispatch_get_main_queue(), ^{
        [[NSNotificationCenter defaultCenter] postNotificationName:@"UIViewControllerStatusBarUpdate" object:@(result == VISIBILITY_HIDDEN)];
    });

    return result;
}

#pragma clang diagnostic pop

@end
